@@ -1,9 +1,9 @@
 package Net::DNS::RR::TSIG;
 
 #
-# $Id: TSIG.pm 1774 2020-03-18 07:49:22Z willem $
+# $Id: TSIG.pm 1779 2020-05-11 09:11:17Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1774 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1779 $)[1];
 
 
 use strict;
@@ -365,47 +365,52 @@ sub create {
 				);
 		}
 
-		croak "Usage:	create $class(keyfile)\n\tcreate $class(keyname, key)";
+		croak "Usage:	create $class( \$keyfile, \@options )";
 
 	} elsif ( scalar(@_) == 1 ) {
-		my $key = shift;				# ( keyname, key )
-		$class->_deprecate('create( $key_name, $key )');
+		$class->_deprecate('create( $keyname, $key )'); # ( keyname, key )
 		return new Net::DNS::RR(
 			name => $karg,
 			type => 'TSIG',
-			key  => $key
+			key  => shift
 			);
 
-	} elsif ( $karg =~ /private$/ ) {			# ( keyfile, options )
-		require File::Spec;
+	} else {
+		require File::Spec;				# ( keyfile, options )
 		require Net::DNS::ZoneFile;
 		my $keyfile = new Net::DNS::ZoneFile($karg);
-		my ( $alg, $key, $junk );
-		while ( $keyfile->_getline ) {
-			( $junk, $alg ) = split if /Algorithm:/;
-			( $junk, $key ) = split if /Key:/;
+		my ( $vol, $dir, $filename ) = File::Spec->splitpath( $keyfile->name );
+
+		$filename =~ m/^K([^+]+)\+\d+\+\d+\./;		# BIND dnssec-keygen
+		my $key = $1;
+
+		if ( $key && $filename =~ /\.key$/ ) {
+			my $keyrr = $keyfile->read;		# BIND dnssec public key
+			croak 'key file incompatible with TSIG' if $keyrr->type ne 'KEY';
+			return new Net::DNS::RR(
+				name	  => $keyrr->name,
+				type	  => 'TSIG',
+				algorithm => $keyrr->algorithm,
+				key	  => $keyrr->key,
+				@_
+				);
 		}
 
-		my ( $vol, $dir, $file ) = File::Spec->splitpath( $keyfile->name );
-		croak 'misnamed private key' unless $file =~ /^K([^+]+)+.+private$/;
-		my $kname = $1;
-		return new Net::DNS::RR(
-			name	  => $kname,
-			type	  => 'TSIG',
-			algorithm => $alg,
-			key	  => $key,
-			@_
-			);
+		my ( $algorithm, $secret, $junk );
+		while ( $keyfile->_getline ) {
+			$key	   = $1 if /^key "([^"]+)"/;	# BIND tsig key
+			$secret	   = $1 if /secret "([^"]+)";/;
+			$algorithm = $1 if /algorithm ([^;]+);/;
+
+			( $junk, $secret )    = split if /^Key:/;	  # BIND dnssec private key
+			( $junk, $algorithm ) = split if /^Algorithm:/;
+		}
 
-	} else {						# ( keyfile, options )
-		require Net::DNS::ZoneFile;
-		my $keyrr = new Net::DNS::ZoneFile($karg)->read;
-		croak 'key file incompatible with TSIG' unless $keyrr->type eq 'KEY';
 		return new Net::DNS::RR(
-			name	  => $keyrr->name,
+			name	  => $key,
 			type	  => 'TSIG',
-			algorithm => $keyrr->algorithm,
-			key	  => $keyrr->key,
+			algorithm => $algorithm,
+			key	  => $secret,
 			@_
 			);
 	}
@@ -664,7 +669,7 @@ The message ID from the header of the original packet.
      $rcode = $tsig->error;
 
 Returns the RCODE covering TSIG processing.  Common values are
-NOERROR, BADSIG, BADKEY, and BADTIME.  See RFC 2845 for details.
+NOERROR, BADSIG, BADKEY, and BADTIME.  See RFC2845 for details.
 
 
 =head2 other
@@ -709,7 +714,7 @@ needed for TSIG-GSS.
 					);
 
 Returns a TSIG RR constructed using the parameters in the specified
-key file, which is assumed to have been generated by dnssec-keygen.
+key file, which is assumed to have been generated by tsig-keygen.
 
 =head2 verify
 
@@ -726,43 +731,34 @@ packet data conforms to the data in the TSIG itself
 
 =head1 TSIG Keys
 
-TSIG keys are symmetric keys generated using dnssec-keygen:
+The TSIG authentication mechanism employs shared secret keys
+to establish a trust relationship between two entities.
 
-	$ dnssec-keygen -a HMAC-SHA1 -b 160 -n HOST <keyname>
+It should be noted that it is possible for more than one key
+to be in use simultaneously between any such pair of entities.
 
-	The key will be stored as a private and public keyfile pair
-	K<keyname>+161+<keyid>.private and K<keyname>+161+<keyid>.key
+TSIG keys are generated using the tsig-keygen utility
+distributed with ISC BIND:
 
-    where
-	<keyname> is the DNS name of the key.
+    tsig-keygen -a HMAC-SHA256 host1-host2.example.
 
-	<keyid> is the (generated) numerical identifier used to
-	distinguish this key.
+Other algorithms may be substituted for HMAC-SHA256 in the above example.
 
-Other algorithms may be substituted for HMAC-SHA1 in the above example.
-
-It is recommended that the keyname be globally unique and incorporate
-the fully qualified domain names of the resolver and nameserver in
-that order. It should be possible for more than one key to be in use
-simultaneously between any such pair of hosts.
-
-Although the formats differ, the private and public keys are identical
-and both should be stored and handled as secret data.
+These keys must be protected in a manner similar to private keys,
+lest a third party masquerade as one of the intended parties
+by forging the message authentication code (MAC).
 
 
 =head1 Configuring BIND Nameserver
 
-The following lines must be added to the /etc/named.conf file:
+The generated key must be added to the /etc/named.conf configuration
+or a separate file introduced by the $INCLUDE directive:
 
-    key <keyname> {
-	algorithm HMAC-SHA1;
-	secret "<keydata>";
+    key "host1-host2.example. {
+	algorithm hmac-sha256;
+	secret "Secret+known+only+by+participating+entities=";
     };
 
-<keyname> is the name of the key chosen when the key was generated.
-
-<keydata> is the key string extracted from the generated key file.
-
 
 =head1 ACKNOWLEDGMENT
 
@@ -789,7 +785,7 @@ Copyright (c)2000,2001 Michael Fuhr.
 
 Portions Copyright (c)2002,2003 Chris Reinhardt.
 
-Portions Copyright (c)2013 Dick Franks.
+Portions Copyright (c)2013,2020 Dick Franks.
 
 All rights reserved.
 
