@@ -1,9 +1,9 @@
 package Net::DNS::Resolver::Base;
 
 #
-# $Id: Base.pm 1748 2019-07-15 07:57:00Z willem $
+# $Id: Base.pm 1784 2020-05-24 19:27:13Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1748 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1784 $)[1];
 
 
 #
@@ -26,7 +26,7 @@ our $VERSION = (qw$LastChangedRevision: 1748 $)[1];
 # [Revised March 2016, June 2018]
 
 
-use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.32; 1';
+use constant USE_SOCKET_IP => defined eval 'use IO::Socket::IP 0.38; 1';
 
 use constant IPv6 => USE_SOCKET_IP;
 
@@ -45,6 +45,7 @@ use strict;
 use warnings;
 use integer;
 use Carp;
+use IO::File;
 use IO::Select;
 use IO::Socket;
 
@@ -188,8 +189,7 @@ sub _read_config_file {			## read resolver config file
 	my $self = shift;
 	my $file = shift;
 
-	my $filehandle;
-	open( $filehandle, '<', $file ) or croak "$file: $!";
+	my $filehandle = new IO::File( $file, '<' ) or croak "$file: $!";
 
 	my @nameserver;
 	my @searchlist;
@@ -430,11 +430,12 @@ sub _send_tcp {
 	my $timeout = $self->{tcp_timeout};
 
 	foreach my $ip (@ns) {
-		my $socket = $self->_create_tcp_socket($ip) || next;
-		my $select = IO::Select->new($socket);
-
 		$self->_diag( 'tcp send', "[$ip]" );
 
+		my $socket = $self->_create_tcp_socket($ip);
+		$self->errorstring($!);
+		my $select = IO::Select->new( $socket || next );
+
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
 
@@ -565,10 +566,12 @@ sub _bgsend_tcp {
 	my $tcp_packet = pack 'n a*', length($packet_data), $packet_data;
 
 	foreach my $ip ( $self->nameservers ) {
-		my $socket = $self->_create_tcp_socket($ip) || next;
-
 		$self->_diag( 'bgsend', "[$ip]" );
 
+		my $socket = $self->_create_tcp_socket($ip);
+		$self->errorstring($!);
+		next unless $socket;
+
 		$socket->blocking(0);
 		$socket->send($tcp_packet);
 		$self->errorstring($!);
@@ -636,14 +639,14 @@ sub bgbusy {
 
 
 sub bgisready {				## historical
-	_deprecate('prefer bgbusy()');				# uncoverable pod
+	_deprecate('prefer  ! bgbusy(...)');			# uncoverable pod
 	!&bgbusy;
 }
 
 
 sub bgread {
 	while (&bgbusy) {					# side effect: TCP retry
-		IO::Select->new( $_[1] )->can_read(0.02);	# use 3 orders of magnitude less CPU
+		IO::Select->new( $_[1] )->can_read(0.02);	# reduce my CPU usage by 3 orders of magnitude
 	}
 	&_bgread;
 }
@@ -716,7 +719,7 @@ sub axfr {				## zone transfer
 			unless ( scalar @rr ) {
 				my $reply;			# refill @rr
 				( $reply, $verify ) = $self->_axfr_next( $select, $verify );
-				@rr = $reply->answer;
+				@rr = $reply->answer if $reply;
 			}
 
 			return $rr;
@@ -756,15 +759,16 @@ sub _axfr_start {
 	my $content = $request->data;
 	my $TCP_msg = pack 'n a*', length($content), $content;
 
-	$self->_diag("axfr_start( $dname @class )");
+	$self->_diag("axfr( $dname @class )");
 
 	my ( $select, $reply, $rcode );
 	foreach my $ns ( $self->nameservers ) {
-		my $socket = $self->_create_tcp_socket($ns) || next;
+		$self->_diag("axfr send [$ns]");
 
-		$self->_diag("axfr_start nameserver [$ns]");
+		my $socket = $self->_create_tcp_socket($ns);
+		$self->errorstring($!);
+		$select = IO::Select->new( $socket || next );
 
-		$select = IO::Select->new($socket);
 		$socket->send($TCP_msg);
 		$self->errorstring($!);
 
@@ -1005,25 +1009,25 @@ sub dnssec {
 
 sub force_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v6};
 	$self->{force_v6} = $value ? do { $self->{force_v4} = 0; 1 } : 0;
 }
 
 sub force_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{force_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{force_v4};
 	$self->{force_v4} = $value ? do { $self->{force_v6} = 0; 1 } : 0;
 }
 
 sub prefer_v6 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v6};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v6};
 	$self->{prefer_v6} = $value ? do { $self->{prefer_v4} = 0; 1 } : 0;
 }
 
 sub prefer_v4 {
 	my $self = shift;
-	my $value = scalar(@_) ? shift() : $self->{prefer_v4};
+	my $value = scalar(@_) ? $_[0] : $self->{prefer_v4};
 	$self->{prefer_v4} = $value ? do { $self->{prefer_v6} = 0; 1 } : 0;
 }
 
